
# q-expansion of newform 1456.2.a.j, downloaded from the LMFDB on 06 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1456
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [0], [-1], [-3], [-1], [4], [-2], [-1], [4], [-9], [3], [-5], [-4], [-9], [-4], [-10], [5], [-11], [16], [11], [5], [4], [-2], [-7], [-9], [2], [0], [14], [-15], [-5], [0], [-22], [20], [-13], [4], [17], [12], [8], [18], [-14], [11], [8], [-20], [15], [-4], [6], [-19], [12], [-16], [-21], [6], [2], [9], [12], [-24], [5], [-15], [22], [-8], [-23], [-22], [0], [6], [10], [23], [21], [33], [-8], [10], [-1], [-20], [24], [4], [32], [1], [-8], [-8], [-36], [10], [21], [25], [-26], [34], [-14], [4], [-22], [6], [12], [-12], [-4], [32], [-14], [-28], [-17], [24], [-34], [2], [-39], [-30], [16], [11], [-7], [-3], [2], [18], [6], [-30], [13], [-22], [26], [-43], [-42], [-14], [2], [-33], [-4], [-28], [-20], [14], [-44], [-9], [-13], [-19], [28], [-20], [-15], [-14], [18], [-36], [16], [-12], [-37], [38], [-37], [-13], [-30], [8], [27], [-10], [52], [6], [43], [-12], [-6], [-15], [-16], [2], [-9], [-24], [1], [-42], [-30], [10], [-46], [-52], [-35], [39], [48], [-42], [-32], [22], [14], [51], [38], [24], [33], [49], [20], [54], [-52], [30], [-53], [-8], [-18], [4], [27], [-18], [-52], [-17], [-2], [3], [-22], [-11], [12], [26], [-48], [20], [28], [11], [-16], [-47], [30], [62], [22], [-54], [56], [-6], [6], [36], [9], [-64], [-48], [-14], [-30], [-30], [52], [58], [-10], [-47], [-67], [-15], [29], [32], [9], [3], [59], [20], [-38], [-27], [-50], [-61], [20], [-58], [63], [40], [72], [-75], [56], [-9], [-36], [44], [28], [-21], [-1], [-21], [-43], [15], [60], [-24], [-9], [19], [4], [18], [-9], [14], [23], [2], [-19], [-35], [53], [-18], [10], [70], [-61], [48], [66], [-38], [41], [81], [64], [-50], [-74], [-11], [-21], [56], [-63], [-57], [-21], [32], [13], [55], [59], [32], [-50], [40], [76], [-29], [12], [-12], [66], [58], [2], [-50], [52], [-9], [-7], [-6], [-11], [-20], [54], [-59], [82], [-22], [-62], [-22], [-36], [-9], [-85], [-50], [-14], [27], [48], [56], [-68], [72], [-14], [-74], [26], [-28], [62], [26], [-43], [10], [-30], [48], [-36], [50], [-30], [-63], [-61], [-32], [73], [-46], [54], [46], [1], [-64], [64], [26], [-63], [32], [-10], [-36], [-5], [66], [-38], [53], [74], [-42], [30], [-35], [26], [-27], [-3], [-84], [30], [36], [-75], [37], [58], [-52], [-71], [-86], [-6], [4], [81], [-14], [-69], [20], [46], [6], [75], [-31], [-95], [-89], [16], [-14], [-31], [-42], [-41], [66], [-72], [-32], [56], [-92], [33], [79], [-56], [-100], [80], [-88], [68], [48], [-48], [40], [-10], [67], [4], [36], [38], [-54], [27], [-36], [-30], [19], [85], [-36], [-62], [22], [-20], [-50], [4], [93], [40], [48], [39], [-20], [68], [-60], [50], [-28], [-6], [68], [83], [90], [62], [-56], [-13], [86], [106], [-70], [36], [12], [26], [60], [22], [-36], [14], [-10]]
